/* cgenerator.cpp -- cgenerator class */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Copyright (C) 2007  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


      
#include "cgenerator.h" 


namespace faudes {


/***********************************************************************************
 *
 * implementation of AttributeCFlags 
 *
 */

//Write(rTw)
// Note: you should write attributes in a section, so that
// the AttributeVoid read method can detect and skip them.
// Here, we make an execption of the rule ...
void AttributeCFlags::Write(TokenWriter& rTw) const {
  if(!IsDefault()) {
    Token token;
    // if no other flags used, write option string
    if( (mFlags & ~mAllCFlags) == 0 ) {
      std::string option;
      if((mDefCFlags & mControllableFlag) != (mFlags & mControllableFlag)) {
        if(Controllable()) option = option+"C";
        else option = option+"c";
      }
      if((mDefCFlags & mObservableFlag) != (mFlags & mObservableFlag)) {
        if(Observable()) option = option+"O";
        else option = option+"o";
      }
      if((mDefCFlags & mForcibleFlag) != (mFlags & mForcibleFlag)) {
        if(Forcible()) option = option+"F";
        else option = option+"f";
      }
      token.SetOption(option);
    } 
    // if other flags used, write hex
    else { 
      token.SetInteger16(mFlags);
    }
    rTw << token;
  }
}

//Write()
void AttributeCFlags::Write(void) const {
  TokenWriter rTw(TokenWriter::Stdout);
  Write(rTw);
}



//ToString()
std::string AttributeCFlags::ToString(void) const {
  return ToStringInteger16(mFlags);
}

/** @file cgenerator.cpp Classes TcGenerator, cGenerator and AttributeCFlags */


//Read(rTr)
void AttributeCFlags::Read(TokenReader& rTr) {
  mFlags=mDefCFlags;
  Token token;
  rTr.Peek(token);
  if(token.Type()==Token::Integer16) { 
    rTr.Get(token);
    mFlags=token.IntegerValue();
    return;
  } 
  if(token.Type()==Token::Option) { 
    rTr.Get(token);
    std::string option=token.StringValue();
    if(option.find( 'C', 0) != std::string::npos) SetControllable();
    if(option.find( 'c', 0) != std::string::npos) ClrControllable();
    if(option.find( 'O', 0) != std::string::npos) SetObservable();
    if(option.find( 'o', 0) != std::string::npos) ClrObservable();
    if(option.find( 'F', 0) != std::string::npos) SetForcible();
    if(option.find( 'f', 0) != std::string::npos) ClrForcible();
    return;
  }
}




} // end namespace
