------------------------------------------------------------------------------
-- generate_models.lua --
--   generate models for n forks and n philosophers with
--   k intermediate states for the WODES'08 "dining philosophers"
--   benchmark example.
-- The models are written to the directory "phil_monolithic/data".
------------------------------------------------------------------------------

  --################################################
  --############# configure parameters #############
  --################################################
  
  -- number of philosophers and forks
  n = 3
  
  -- number of intermediate states
  k = 4
  
  --###############################################
  --############# "include" functions #############
  --###############################################
  
  -- make function Philosopher() defined in generate_philosopher.lua
  -- available to our script
  dofile('generate_philosopher.lua')
  
  -- make function Fork() defined in generate_fork.lua
  -- available to our script
  dofile('generate_fork.lua')

  --################################################
  --### create models of philosophers and forks: ###
  --################################################

  -- report to console
  print('***************************************************************')
  print('*************** generate_models.lua ***************************')
  print('***************************************************************')
  print('***')
  print(string.format('*** Generating models of n=%d forks and n=%d philosophers with k=%d intermediate states...',n,n,k))
  
   -- catch invalid numbers
   if(k<1) then print('*** ERROR: k has to be 1 at minimum!') end
   if(n<2) then print('*** ERROR: n has to be 2 at minimum!') end

  
  -- Initial step: create philosopher 1 and fork between philosopher n and 1
  
  phil=Philosopher(1,k)
  phil:Write(string.format('data/Philosopher_n1_k%d.gen',k))


  fork=Fork(n,1)
  fork:Write(string.format('data/Fork_n%d_n1.gen',n))

  -- loop over n
  for i=2,n do
    phil=Philosopher(i,k)
    phil:Write(string.format('data/Philosopher_n%d_k%d.gen',i,k))

    fork=Fork(i-1,i)
    fork:Write(string.format('data/Fork_n%d_n%d.gen',i-1,i))
  end
  
  print('***')
  print('*** ...done.')
  print('***************************************************************')
